/*
 * File:   util.h
 *
 * Contains some useful defines and auxiliary methods.
 *
 * Author: Jan Goedgebeur (Jan.Goedgebeur@UGent.be)
 *
 */

#ifndef _UTIL_H
#define	_UTIL_H

/****************************Defines for timing********************************/

#include <time.h>
#if !defined(CLK_TCK) && !defined(_SC_CLK_TCK)
#include <unistd.h>
#endif
#if !defined(CLK_TCK) && defined(_SC_CLK_TCK)
#define CLK_TCK sysconf(_SC_CLK_TCK)
#endif
#if !defined(CLK_TCK) && defined(CLOCKS_PER_SEC)
#define CLK_TCK CLOCKS_PER_SEC
#endif
#if !defined(CLK_TCK)
#define CLK_TCK 60     /* If the CPU time stated by the program appears
                      to be out by a constant ratio, the most likely
                      explanation is that the code got to this point but
                      60 is the wrong guess.  Another common value is 100. */
#endif


/**********************************Defines*************************************/

#define BIT(i) (1ULL << (i))
#define MAX(a, b) (((a) >= (b)) ? (a) : (b))

//In this specific example MAXN is certainly not limited to 64
#define MAXN 64 /* Should be 32 on a 32-bit machine */

//Actually MAXN-1 is enough for the number of neighbours
typedef unsigned char GRAPH[MAXN][MAXN];
typedef unsigned char ADJACENCY[MAXN];

//Slightly faster when storing this in a global variable instead of passing as a param
GRAPH graph;
ADJACENCY adj;

//nv of the graph
unsigned char number_of_vertices = 0;


/***************************Useful auxiliary methods***************************/


static void 
add_edge(unsigned char adj[], unsigned char v, unsigned char w) {
    graph[v][adj[v]] = w;
    graph[w][adj[w]] = v;
    adj[v]++;
    adj[w]++;
}

/******************************************************************************/

/**
 * Decodes the code (which is in multicode format) of a graph.
 */
static void 
decode_multicode(unsigned char code[], int codelength) {
    int i;

    if(number_of_vertices != code[0]) {
        fprintf(stderr, "Error: Wrong number of vertices: expected %d while found %d vertices \n", number_of_vertices, code[0]);
        exit(1);
    }
    //graph[0][0] = number_of_vertices;

    for(i = 0; i < number_of_vertices; i++) {
        adj[i] = 0;
    }

    int currentvertex = 1;
    //Codelength
    for(i = 1; i < codelength; i++) {
        if(code[i] == 0) {
            currentvertex++;
        } else {
            add_edge(adj, currentvertex - 1, code[i] - 1);
        }
    }

}

/******************************************************************************/

static void 
printgraph(GRAPH graph, ADJACENCY adj, int nv) {
    int i, j;
    fprintf(stderr, "Printing graph: \n");
    for(i = 0; i < nv; i++) {
        fprintf(stderr, "%d :", i);
        for(j = 0; j < adj[i]; j++)
            fprintf(stderr, " %d", graph[i][j]);
        fprintf(stderr, "\n");
    }
}


#endif	/* _UTIL_H */